package getenv;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.tools.ant.taskdefs.Execute;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.externaltools.internal.variable.DefaultVariableExpander;
import org.eclipse.ui.externaltools.internal.variable.ExpandVariableContext;


public class GetEnvExpander extends DefaultVariableExpander {
	private Map envMap;

    public String getText(String varTag, String varValue, ExpandVariableContext context) throws CoreException {
        if ("env".equals(varTag)) { //$NON-NLS-1$
            String env = getEnv(varValue);
            if (env == null)
                throwNotFoundException(varValue, "envNotFound"); //$NON-NLS-1$
            return env;
        }
        else if ("which".equals(varTag)) { //$NON-NLS-1$
            String cmdPath = which(varValue);
            if (cmdPath == null)
                throwNotFoundException(varValue, "commandNotFound"); //$NON-NLS-1$
            return cmdPath;
        }
        return null;
    }

    private void throwNotFoundException(String varValue, String messageId) throws CoreException {
        String message = Messages.getString("GetEnvExpander." + messageId, varValue); //$NON-NLS-1$
        throw new CoreException(new Status(IStatus.ERROR, "getenv", IStatus.OK, message, null)); //$NON-NLS-1$
    }

	public IPath getPath(String varTag, String varValue, ExpandVariableContext context) throws CoreException {
        String text = getText(varTag, varValue, context);
        if (text == null)
            return null;
        return new Path(text);
	}

	String getEnv(String name) {
        if (envMap == null) {
			envMap = new HashMap();
			List envs = Execute.getProcEnvironment();
			for (int i = 0; i < envs.size(); ++i ) {
                String env = (String) envs.get(i);
                int sep = env.indexOf('=');
                if (sep != -1) {
                    envMap.put(env.substring(0, sep).toUpperCase(), env.substring(sep + 1));
                }
			}
		}
		return (String) envMap.get(name.toUpperCase());
	}

	public String which(String cmd) {
        return which(cmd, getEnv("PATH"), getEnv("PATHEXT")); //$NON-NLS-1$ //$NON-NLS-2$
	}

    private String[] getCommandCandidates(String cmd, String pathExt) {
        if (cmd.indexOf(".") != -1 || pathExt == null) //$NON-NLS-1$
            return new String[] { cmd };

        StringTokenizer st = new StringTokenizer(pathExt, File.pathSeparator);
        String[] candidates = new String[st.countTokens() + 1];
        for (int i = 0; i < candidates.length - 1; ++i) {
            candidates[i] = cmd + st.nextToken().toLowerCase();
        }
        candidates[candidates.length - 1] = cmd;
        return candidates;
    }

	String which(String cmd, String path, String pathExt) {
        if (cmd == null || path == null)
            return null;

        String[] candidates = getCommandCandidates(cmd, pathExt);
        StringTokenizer st = new StringTokenizer(path, File.pathSeparator);
        
        while(st.hasMoreTokens()) {
            String pathElement = st.nextToken();
            for (int i = 0; i < candidates.length; ++i) {
                File file = new File(pathElement, candidates[i]);
                if (file.exists() && file.isFile()) {
                    try {
                        return file.getCanonicalPath();
                    } catch (IOException e) {
                        continue;
                    }
                }
            }
        }
        return null;
	}
}
